#include <string.h>
#include <alloca.h>
#include <SD.h>

// ConfigFile.h
// Maurice Castro
// Tue Mar  5 10:23:25 EST 2013

#include "ConfigFile.h"

boolean CfgFile::initialised = false;

// #define DEBUG 1

boolean CfgFile::begin(int ss, int cs)
{
  // cs chip select pin - usually 4
  // ss pin - usually 10
  // only need to execute this once 
  if (initialised)
    return true;
  pinMode(ss, OUTPUT);
  if (SD.begin(cs))
  {
    initialised = true;
    return true;
  }
  return false;
}

boolean CfgFile::open()
{
  if (!initialised)
    return false;
  if (cfg = SD.open(CfgFile::cfg_filename, FILE_READ))
  {
    opened = true;
    return true;
  }
  return false;
}

char CfgFile::read()
{
  return CfgFile::cfg.read();
}

boolean CfgFile::available()
{
  return CfgFile::cfg.available();
}

void CfgFile::close()
{
  CfgFile::cfg.close();
}

char *CfgFile::readtok(char *s, int maxidx)
{
  if (!opened)
  {
#ifdef DEBUG
    Serial.println("not open for readtok");
#endif
    return NULL;
  }
  int i = 0;
  int state = 0; 
  while (available())
  {
    char c = read();
#ifdef DEBUG
    Serial.print(state);
    Serial.print(":");
    Serial.print(c);
    Serial.println();
#endif
    switch (state)
    {
      case 0:
        // absorb leading white space
        switch (c)
        {
          case '#':
            state = 2;
            break;
          case ' ':
          case '\t':
          case '\n':
            break;
          default:
            if (i < maxidx)
            {
              s[i] = c;
              s[i+1] = '\0';
            }
            state = 1;
            i++;
        }
        break;
      case 1:
        // aquire new chars in token
        switch (c)
        {
          case '#':
            state = 2;
            break;
          case '\n':
          case ' ':
          case '\t':
            return s;
          default:
            if (i < maxidx)
            {
              s[i] = c;
              s[i+1] = '\0';
            }
            i++;
        }
        break;
      case 2:
        // in comment
        switch (c)
        {
          case '\n':
            state = 0;
          default:
            break;
        }
        break;
    }
  }
#ifdef DEBUG
  Serial.println("Unavailable");
#endif
  return NULL;
}

CfgFile::CfgFile(char *filename)
{
  CfgFile::cfg_filename = filename;
  CfgFile::opened = false;
}

char *CfgFile::keyvalue(char *key, char *value, int maxlen)
{
#ifdef DEBUG
  Serial.print(key);
#endif
  char *s = (char *) alloca(strlen(key) + 1);
  if (!s)
  {
#ifdef DEBUG
    Serial.println(" Alloca failed");
#endif
    return NULL;
  }
  if (!opened)
  {
#ifdef DEBUG
    Serial.println(" Not open");
#endif
    return NULL;
  }
  cfg.seek(0);
  while (available())
  {
    char *k = readtok(s, strlen(key)+1);
    if (!k)
    {
#ifdef DEBUG
      Serial.println(" readtok returned null");
#endif
      return NULL;
    }
    value[maxlen-1] = 0;
    char *r = readtok(value, maxlen-1);
#ifdef DEBUG
    Serial.print(" = ");
    Serial.println(r);
#endif
    if (!strcasecmp(k, key))
    {
      return(r);
    }
  }
  return NULL;
}
